@@ -20,13 +20,13 @@
     The `Pygments tip`_ is installable with ``easy_install Pygments==dev``.
 
     .. _Pygments tip:
-       http://dev.pocoo.org/hg/pygments-main/archive/tip.tar.gz#egg=Pygments-dev
+       http://bitbucket.org/birkenfeld/pygments-main/get/tip.zip#egg=Pygments-dev
 
-    :copyright: Copyright 2006-2010 by the Pygments team, see AUTHORS.
+    :copyright: Copyright 2006-2014 by the Pygments team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
-__version__ = '1.3'
+__version__ = '2.0'
 __docformat__ = 'restructuredtext'
 
 __all__ = ['lex', 'format', 'highlight']
@@ -43,7 +43,7 @@ def lex(code, lexer):
     """
     try:
         return lexer.get_tokens(code)
-    except TypeError, err:
+    except TypeError as err:
         if isinstance(err.args[0], str) and \
            'unbound method get_tokens' in err.args[0]:
             raise TypeError('lex() argument must be a lexer instance, '
@@ -67,7 +67,7 @@ def format(tokens, formatter, outfile=None):
             return realoutfile.getvalue()
         else:
             formatter.format(tokens, outfile)
-    except TypeError, err:
+    except TypeError as err:
         if isinstance(err.args[0], str) and \
            'unbound method format' in err.args[0]:
             raise TypeError('format() argument must be a formatter instance, '
